unit Main;

interface

uses
  Windows, Classes, Controls, Forms,
  StdCtrls, ExtCtrls, Buttons, ComCtrls, SendMail,
  Menus, ImgList, Dialogs;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    SendBtn: TSpeedButton;
    AbortBtn: TSpeedButton;
    FromPanel: TPanel;
    ToPanel: TPanel;
    CCPanel: TPanel;
    BCCPanel: TPanel;
    SubjectPanel: TPanel;
    AttachPanel: TPanel;
    AttachView: TListView;
    FromAddressEdit: TEdit;
    ToEdit: TEdit;
    CCEdit: TEdit;
    BCCEdit: TEdit;
    SubjectEdit: TEdit;
    GroupBox1: TGroupBox;
    Memo1: TMemo;
    PriorityBox: TComboBox;
    AttachBtn: TSpeedButton;
    Label1: TLabel;
    PlainBox: TRadioButton;
    HTMLBox: TRadioButton;
    Panel3: TPanel;
    Panel4: TPanel;
    Panel5: TPanel;
    Panel6: TPanel;
    Panel7: TPanel;
    Panel8: TPanel;
    FromAddressPanel: TPanel;
    Panel9: TPanel;
    FromNameEdit: TEdit;
    Label2: TLabel;
    SMTPHostEdit: TEdit;
    Label3: TLabel;
    SMTPPortEdit: TEdit;
    Panel10: TPanel;
    AttachTypeBox: TCheckBox;
    Label4: TLabel;
    CharsetEdit: TEdit;
    StatusBox: TGroupBox;
    StatusList: TListBox;
    OpenDialog1: TOpenDialog;
    PopupMenu1: TPopupMenu;
    Open1: TMenuItem;
    N1: TMenuItem;
    Add1: TMenuItem;
    Remove1: TMenuItem;
    Print1: TMenuItem;
    SaveBtn: TSpeedButton;
    SaveDialog: TSaveDialog;
    StatusBar1: TStatusBar;
    ProgressBar1: TProgressBar;
    Panel11: TPanel;
    FromCompanyEdit: TEdit;
    SendMail1: TSendMail;
    procedure SendBtnClick(Sender: TObject);
    procedure AbortBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SendMail1Response(Sender: TObject; Code: Integer;
      Response: string);
    procedure SendMail1HostUnreachable(Sender: TObject);
    procedure SendMail1SMTPError(Sender: TObject; Code: Integer;
      Response: string);
    procedure AttachBtnClick(Sender: TObject);
    procedure AttachViewChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure Open1Click(Sender: TObject);
    procedure Remove1Click(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure SendMail1Progress(Sender: TObject; MessageSize,
      BytesSent: Integer; PercentsDone: Byte);
    procedure SendMail1ConnLost(Sender: TObject);
    procedure SendMail1Success(Sender: TObject);
    procedure SendMail1AnyError(Sender: TObject);
  private
    procedure FixButtonState;
    procedure PrepareSendMail;

    procedure LoadSettings;
    procedure StoreSettings;
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Registry, ShellAPI, SysUtils;

const
  REG_SAVE = '\Software\Utilmind Solutions\SendMailDemo';
  REG_SMTPHOST    = 'SMTPHost';
  REG_SMTPPORT    = 'SMTPPort1';
  REG_FROMADDRESS = 'FromAddress';
  REG_FROMNAME    = 'FromName';
  REG_FROMCOMPANY = 'FromOrganization';
  REG_TO          = 'To';
  REG_TOCC        = 'ToCC';
  REG_TOBCC       = 'ToBCC';
  REG_CHARSET     = 'Charset';
  REG_PLAINTEXT   = 'PlainText';
  REG_SUBJECT     = 'Subject';
  REG_MSGBODY     = 'MsgBody';
  REG_PRIORITY    = 'Priority';
  REG_ATTACHTYPE  = 'AttachType';

procedure TForm1.FixButtonState;
begin
  AbortBtn.Enabled := SendMail1.Busy;
  SendBtn.Enabled := not AbortBtn.Enabled;
end;

procedure TForm1.PrepareSendMail;
begin
  with SendMail1 do
   begin
    SMTPHost := SMTPHostEdit.Text;
    SMTPPort := StrToIntDef(SMTPPortEdit.Text, 25);

    FromAddress := FromAddressEdit.Text;
    FromName := FromNameEdit.Text;
    FromOrganization := FromCompanyEdit.Text;
    ToAddr := ToEdit.Text;
    ToCC   := CCEdit.Text;
    ToBCC  := BCCEdit.Text;

    MsgSubject    := SubjectEdit.Text;
    MsgBody       := Memo1.Text;
    MsgContentCharset := CharsetEdit.Text;
    if PlainBox.Checked then
      MsgContentType := ctPlainText
    else
      MsgContentType := ctHTML;
    MsgPriority := TSMTPPriority(PriorityBox.ItemIndex);

    if AttachTypeBox.Checked then
      AttachType := atEmbedded
    else
      AttachType := atNormal;
   end;
end;

procedure TForm1.LoadSettings;
begin
  with TRegistry.Create do
   try
     try
       RootKey := HKEY_CURRENT_USER;
       if OpenKey(REG_SAVE, False) then
        begin
         SMTPHostEdit.Text := ReadString(REG_SMTPHOST);
         SMTPPortEdit.Text := ReadString(REG_SMTPPORT);
         if SMTPPortEdit.Text = '' then
           SMTPPortEdit.Text := '25';

         FromAddressEdit.Text := ReadString(REG_FROMADDRESS);
         FromNameEdit.Text := ReadString(REG_FROMNAME);
         FromCompanyEdit.Text := ReadString(REG_FROMCOMPANY);
         ToEdit.Text := ReadString(REG_TO);
         CCEdit.Text := ReadString(REG_TOCC);
         BCCEdit.Text := ReadString(REG_TOBCC);

         CharsetEdit.Text := ReadString(REG_CHARSET);
         PlainBox.Checked := ReadBool(REG_PLAINTEXT);
         HTMLBox.Checked := not PlainBox.Checked;
         PriorityBox.ItemIndex := ReadInteger(REG_PRIORITY);

         SubjectEdit.Text := ReadString(REG_SUBJECT);
         Memo1.Text := ReadString(REG_MSGBODY);

         AttachTypeBox.Checked := ReadBool(REG_ATTACHTYPE);
        end;
     except
     end;
   finally
     Free;
   end;
end;

procedure TForm1.StoreSettings;
begin
  with TRegistry.Create do
   try
     try
       RootKey := HKEY_CURRENT_USER;
       if OpenKey(REG_SAVE, True) then
        begin
         WriteString(REG_SMTPHOST, SMTPHostEdit.Text);
         WriteString(REG_SMTPPORT, SMTPPortEdit.Text);

         WriteString(REG_FROMADDRESS, FromAddressEdit.Text);
         WriteString(REG_FROMNAME, FromNameEdit.Text);
         WriteString(REG_FROMCOMPANY, FromCompanyEdit.Text);
         WriteString(REG_TO, ToEdit.Text);
         WriteString(REG_TOCC, CCEdit.Text);
         WriteString(REG_TOBCC, BCCEdit.Text);

         WriteString(REG_CHARSET, CharsetEdit.Text);
         WriteBool(REG_PLAINTEXT, PlainBox.Checked);
         WriteInteger(REG_PRIORITY, PriorityBox.ItemIndex);

         WriteString(REG_SUBJECT, SubjectEdit.Text);
         WriteString(REG_MSGBODY, Memo1.Text);

         WriteBool(REG_ATTACHTYPE, AttachTypeBox.Checked);         
        end;
     except
     end;
   finally
     Free;
   end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  PriorityBox.ItemIndex := 2;
  LoadSettings;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  StoreSettings;
end;

procedure TForm1.SendBtnClick(Sender: TObject);
begin
  PrepareSendMail;
  SendMail1.Send;
  FixButtonState;
end;

procedure TForm1.AbortBtnClick(Sender: TObject);
begin
  SendMail1.Abort;
  FixButtonState;  
end;

procedure TForm1.AttachBtnClick(Sender: TObject);
var
  I: Integer;
  WindowStyle: LongInt;
begin
  with OpenDialog1 do
   if Execute then
    begin
     InitialDir := ExtractFilePath(FileName);

     I := Files.Count;
     if I <> 0 then
      begin
       AttachPanel.Visible := True;
       for I := 0 to I - 1 do
        with AttachView.Items.Add do
         begin
          Caption := ExtractFileName(Files[I]);
          SendMail1.Attach.Add(Files[I]); // attach
         end;

       AttachView.Update;
       while ((GetWindowLong(AttachView.Handle, GWL_STYLE) and WS_VSCROLL <> 0) or
              (GetWindowLong(AttachView.Handle, GWL_STYLE) and WS_HSCROLL <> 0)) and
             (AttachPanel.Height < 60) do
        AttachPanel.Height := AttachPanel.Height + 24;
      end;
    end;
end;

procedure TForm1.AttachViewChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  Open1.Enabled := AttachView.Selected <> nil;
  Print1.Enabled := Open1.Enabled;
  Remove1.Enabled := Open1.Enabled;
end;

procedure TForm1.Open1Click(Sender: TObject);
const
  Operations: Array[Boolean] of String = ('open', 'print');
begin
  if AttachView.Selected <> nil then
    ShellExecute(GetDesktopWindow, PChar(Operations[Sender = Print1]),
      PChar(SendMail1.Attach[AttachView.Selected.Index]), nil, nil, SW_SHOWNORMAL);
end;

procedure TForm1.Remove1Click(Sender: TObject);
var
  I: Integer;
begin
  with AttachView do
   if Selected <> nil then
    begin
     for I := Items.Count - 1 downto 0 do
      if Items[I].Selected then
       begin
        SendMail1.Attach.Delete(I);
        Items.Delete(Selected.Index);
       end;
     AttachPanel.Visible := Items.Count <> 0;
    end;
end;

procedure TForm1.SaveBtnClick(Sender: TObject);
begin
  if SaveDialog.Execute then
   begin
    PrepareSendMail;
    SendMail1.SaveToFile(SaveDialog.FileName, True);
   end;
end;

procedure TForm1.SendMail1Response(Sender: TObject; Code: Integer;
  Response: string);
begin
  with StatusList do
   begin
    Items.Add(Response);
    ItemIndex := Items.Count - 1;
   end;
end;

procedure TForm1.SendMail1HostUnreachable(Sender: TObject);
begin
  Application.MessageBox('Specified SMTP server is unreachable.'#13#10#10'Please check your Internet connection'#13#10'or specify another SMTP host or/and port.', 'Host unreachable', MB_OK or MB_ICONSTOP);
  SMTPHostEdit.SetFocus;
end;

procedure TForm1.SendMail1SMTPError(Sender: TObject; Code: Integer;
  Response: string);
begin
  with StatusList do
   begin
    Items.Add('ERROR: ' + Response);
    ItemIndex := Items.Count - 1;
   end;
end;

procedure TForm1.SendMail1Progress(Sender: TObject; MessageSize,
  BytesSent: Integer; PercentsDone: Byte);
begin
  ProgressBar1.Position := PercentsDone;
end;

procedure TForm1.SendMail1ConnLost(Sender: TObject);
begin
  ShowMessage('Connection Lost!');
end;

procedure TForm1.SendMail1Success(Sender: TObject);
begin
  FixButtonState;
  ShowMessage('Message sent!');
end;

procedure TForm1.SendMail1AnyError(Sender: TObject);
begin
  FixButtonState;
end;

end.
